"use strict";

var _CountingSort = _interopRequireDefault(require("../CountingSort"));

var _SortTester = require("../../SortTester");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Complexity constants.
const SORTED_ARRAY_VISITING_COUNT = 60;
const NOT_SORTED_ARRAY_VISITING_COUNT = 60;
const REVERSE_SORTED_ARRAY_VISITING_COUNT = 60;
const EQUAL_ARRAY_VISITING_COUNT = 60;
describe('CountingSort', () => {
  it('should sort array', () => {
    _SortTester.SortTester.testSort(_CountingSort.default);
  });
  it('should sort negative numbers', () => {
    _SortTester.SortTester.testNegativeNumbersSort(_CountingSort.default);
  });
  it('should allow to use specify max/min integer value in array to make sorting faster', () => {
    const visitingCallback = jest.fn();
    const sorter = new _CountingSort.default({
      visitingCallback
    }); // Detect biggest number in array in prior.

    const biggestElement = Math.max(..._SortTester.notSortedArr); // Detect smallest number in array in prior.

    const smallestElement = Math.min(..._SortTester.notSortedArr);
    const sortedArray = sorter.sort(_SortTester.notSortedArr, smallestElement, biggestElement);
    expect(sortedArray).toEqual(_SortTester.sortedArr); // Normally visitingCallback is being called 60 times but in this case
    // it should be called only 40 times.

    expect(visitingCallback).toHaveBeenCalledTimes(40);
  });
  it('should visit EQUAL array element specified number of times', () => {
    _SortTester.SortTester.testAlgorithmTimeComplexity(_CountingSort.default, _SortTester.equalArr, EQUAL_ARRAY_VISITING_COUNT);
  });
  it('should visit SORTED array element specified number of times', () => {
    _SortTester.SortTester.testAlgorithmTimeComplexity(_CountingSort.default, _SortTester.sortedArr, SORTED_ARRAY_VISITING_COUNT);
  });
  it('should visit NOT SORTED array element specified number of times', () => {
    _SortTester.SortTester.testAlgorithmTimeComplexity(_CountingSort.default, _SortTester.notSortedArr, NOT_SORTED_ARRAY_VISITING_COUNT);
  });
  it('should visit REVERSE SORTED array element specified number of times', () => {
    _SortTester.SortTester.testAlgorithmTimeComplexity(_CountingSort.default, _SortTester.reverseArr, REVERSE_SORTED_ARRAY_VISITING_COUNT);
  });
});